#!/usr/bin/env python

"""
Accessor classes for indexing.

Copyright (C) 2011 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

class Accessor:

    "An abstract accessor."

    def reset(self):
        pass

    def convert(self, term):
        return term

    def get_key(self, record):
        return record

class DelimitedRecord(Accessor):

    "An accessor using a delimiter to split a record."

    def __init__(self, fields=None, delimiter=None):

        """
        Initialise the accessor using a sequence of 'fields' indicating the
        columns in each record that provide the values in the eventual compound
        key provided by each record, along with a 'delimiter' indicating how
        such columns are identified.
        """

        self.fields = fields or [0]
        self.delimiter = delimiter

    def get_key(self, record):
        values = record.split(self.delimiter)
        return [values[field] for field in self.fields]

class Converted(Accessor):

    "Conversion of keys."

    def __init__(self, accessor, converters=None):

        "Wrap the given 'accessor' with the given 'converters'."

        self.accessor = accessor
        self.converters = converters

    def get_converter(self, converter):
        return converter and converter.convert or (lambda x: x)

    def reset(self):
        for converter in self.converters:
            if converter:
                converter.reset()

    def convert(self, term):
        converters = map(self.get_converter, self.converters)
        return [convert(value) for convert, value in zip(converters, term)]

    def get_key(self, record):
        key = self.accessor.get_key(record)
        return self.convert(key)

# vim: tabstop=4 expandtab shiftwidth=4
